@@ -6,22 +6,22 @@ import (
 	"net/url"
 	"os"
 
+	"github.com/crowdsecurity/crowdsec/pkg/apiclient"
+	"github.com/crowdsecurity/crowdsec/pkg/csconfig"
+	"github.com/crowdsecurity/crowdsec/pkg/cwhub"
+	"github.com/crowdsecurity/crowdsec/pkg/cwversion"
+	"github.com/crowdsecurity/crowdsec/pkg/fflag"
+	"github.com/crowdsecurity/crowdsec/pkg/models"
+	"github.com/crowdsecurity/crowdsec/pkg/types"
 	"github.com/go-openapi/strfmt"
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
-
-	"github.com/crowdsecurity/crowdsec/pkg/apiclient"
-	"github.com/crowdsecurity/crowdsec/pkg/csconfig"
-	"github.com/crowdsecurity/crowdsec/pkg/cwhub"
-	"github.com/crowdsecurity/crowdsec/pkg/cwversion"
-	"github.com/crowdsecurity/crowdsec/pkg/models"
 )
 
-var CAPIURLPrefix string = "v2"
-var CAPIBaseURL string = "https://api.crowdsec.net/"
-var capiUserPrefix string
+const CAPIBaseURL string = "https://api.crowdsec.net/"
+const CAPIURLPrefix = "v3"
 
 func NewCapiCmd() *cobra.Command {
 	var cmdCapi = &cobra.Command{
@@ -41,6 +41,16 @@ func NewCapiCmd() *cobra.Command {
 		},
 	}
 
+	cmdCapi.AddCommand(NewCapiRegisterCmd())
+	cmdCapi.AddCommand(NewCapiStatusCmd())
+
+	return cmdCapi
+}
+
+func NewCapiRegisterCmd() *cobra.Command {
+	var capiUserPrefix string
+	var outputFile string
+
 	var cmdCapiRegister = &cobra.Command{
 		Use:               "register",
 		Short:             "Register to Central API (CAPI)",
@@ -53,9 +63,9 @@ func NewCapiCmd() *cobra.Command {
 				log.Fatalf("unable to generate machine id: %s", err)
 			}
 			password := strfmt.Password(generatePassword(passwordLength))
-			apiurl, err := url.Parse(CAPIBaseURL)
+			apiurl, err := url.Parse(types.CAPIBaseURL)
 			if err != nil {
-				log.Fatalf("unable to parse api url %s : %s", CAPIBaseURL, err)
+				log.Fatalf("unable to parse api url %s : %s", types.CAPIBaseURL, err)
 			}
 			_, err = apiclient.RegisterClient(&apiclient.Config{
 				MachineID:     capiUser,
@@ -66,7 +76,7 @@ func NewCapiCmd() *cobra.Command {
 			}, nil)
 
 			if err != nil {
-				log.Fatalf("api client register ('%s'): %s", CAPIBaseURL, err)
+				log.Fatalf("api client register ('%s'): %s", types.CAPIBaseURL, err)
 			}
 			log.Printf("Successfully registered to Central API (CAPI)")
 
@@ -82,7 +92,10 @@ func NewCapiCmd() *cobra.Command {
 			apiCfg := csconfig.ApiCredentialsCfg{
 				Login:    capiUser,
 				Password: password.String(),
-				URL:      CAPIBaseURL,
+				URL:      types.CAPIBaseURL,
+			}
+			if fflag.PapiClient.IsEnabled() {
+				apiCfg.PapiURL = types.PAPIBaseURL
 			}
 			apiConfigDump, err := yaml.Marshal(apiCfg)
 			if err != nil {
@@ -106,8 +119,11 @@ func NewCapiCmd() *cobra.Command {
 	if err := cmdCapiRegister.Flags().MarkHidden("schmilblick"); err != nil {
 		log.Fatalf("failed to hide flag: %s", err)
 	}
-	cmdCapi.AddCommand(cmdCapiRegister)
 
+	return cmdCapiRegister
+}
+
+func NewCapiStatusCmd() *cobra.Command {
 	var cmdCapiStatus = &cobra.Command{
 		Use:               "status",
 		Short:             "Check status with the Central API (CAPI)",
@@ -159,14 +175,13 @@ func NewCapiCmd() *cobra.Command {
 			}
 			log.Infof("Loaded credentials from %s", csConfig.API.Server.OnlineClient.CredentialsFilePath)
 			log.Infof("Trying to authenticate with username %s on %s", csConfig.API.Server.OnlineClient.Credentials.Login, apiurl)
-			_, err = Client.Auth.AuthenticateWatcher(context.Background(), t)
+			_, _, err = Client.Auth.AuthenticateWatcher(context.Background(), t)
 			if err != nil {
 				log.Fatalf("Failed to authenticate to Central API (CAPI) : %s", err)
 			}
 			log.Infof("You can successfully interact with Central API (CAPI)")
 		},
 	}
-	cmdCapi.AddCommand(cmdCapiStatus)
 
-	return cmdCapi
+	return cmdCapiStatus
 }
