@@ -87,7 +87,19 @@ cscli dashboard remove
 		},
 	}
 
+	cmdDashboard.AddCommand(NewDashboardSetupCmd())
+	cmdDashboard.AddCommand(NewDashboardStartCmd())
+	cmdDashboard.AddCommand(NewDashboardStopCmd())
+	cmdDashboard.AddCommand(NewDashboardShowPasswordCmd())
+	cmdDashboard.AddCommand(NewDashboardRemoveCmd())
+
+	return cmdDashboard
+}
+
+
+func NewDashboardSetupCmd() *cobra.Command {
 	var force bool
+
 	var cmdDashSetup = &cobra.Command{
 		Use:               "setup",
 		Short:             "Setup a metabase container.",
@@ -192,8 +204,10 @@ cscli dashboard setup -l 0.0.0.0 -p 443 --password <password>
 	//cmdDashSetup.Flags().StringVarP(&metabaseUser, "user", "u", "crowdsec@crowdsec.net", "metabase user")
 	cmdDashSetup.Flags().StringVar(&metabasePassword, "password", "", "metabase password")
 
-	cmdDashboard.AddCommand(cmdDashSetup)
+	return cmdDashSetup
+}
 
+func NewDashboardStartCmd() *cobra.Command {
 	var cmdDashStart = &cobra.Command{
 		Use:               "start",
 		Short:             "Start the metabase container.",
@@ -212,8 +226,10 @@ cscli dashboard setup -l 0.0.0.0 -p 443 --password <password>
 			log.Infof("url : http://%s:%s", metabaseListenAddress, metabaseListenPort)
 		},
 	}
-	cmdDashboard.AddCommand(cmdDashStart)
+	return cmdDashStart
+}
 
+func NewDashboardStopCmd() *cobra.Command {
 	var cmdDashStop = &cobra.Command{
 		Use:               "stop",
 		Short:             "Stops the metabase container.",
@@ -226,8 +242,11 @@ cscli dashboard setup -l 0.0.0.0 -p 443 --password <password>
 			}
 		},
 	}
-	cmdDashboard.AddCommand(cmdDashStop)
+	return cmdDashStop
+}
 
+
+func NewDashboardShowPasswordCmd() *cobra.Command {
 	var cmdDashShowPassword = &cobra.Command{Use: "show-password",
 		Short:             "displays password of metabase.",
 		Args:              cobra.ExactArgs(0),
@@ -240,7 +259,12 @@ cscli dashboard setup -l 0.0.0.0 -p 443 --password <password>
 			log.Printf("'%s'", m.Config.Password)
 		},
 	}
-	cmdDashboard.AddCommand(cmdDashShowPassword)
+	return cmdDashShowPassword
+}
+
+
+func NewDashboardRemoveCmd() *cobra.Command {
+	var force bool
 
 	var cmdDashRemove = &cobra.Command{
 		Use:               "remove",
@@ -303,9 +327,8 @@ cscli dashboard remove --force
 	}
 	cmdDashRemove.Flags().BoolVarP(&force, "force", "f", false, "Remove also the metabase image")
 	cmdDashRemove.Flags().BoolVarP(&forceYes, "yes", "y", false, "force  yes")
-	cmdDashboard.AddCommand(cmdDashRemove)
 
-	return cmdDashboard
+	return cmdDashRemove
 }
 
 func passwordIsValid(password string) bool {
