@@ -6,7 +6,7 @@ import (
 	"syscall"
 	"time"
 
-	"github.com/coreos/go-systemd/daemon"
+	"github.com/coreos/go-systemd/v22/daemon"
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
 	"gopkg.in/tomb.v2"
@@ -18,7 +18,7 @@ import (
 	"github.com/crowdsecurity/crowdsec/pkg/types"
 )
 
-//nolint: deadcode,unused // debugHandler is kept as a dev convenience: it shuts down and serialize internal state
+//nolint:deadcode,unused // debugHandler is kept as a dev convenience: it shuts down and serialize internal state
 func debugHandler(sig os.Signal, cConfig *csconfig.Config) error {
 	var (
 		tmpFile string
@@ -54,23 +54,11 @@ func reloadHandler(sig os.Signal) (*csconfig.Config, error) {
 	crowdsecTomb = tomb.Tomb{}
 	pluginTomb = tomb.Tomb{}
 
-	cConfig, err := csconfig.NewConfig(flags.ConfigFile, flags.DisableAgent, flags.DisableAPI)
+	cConfig, err := LoadConfig(flags.ConfigFile, flags.DisableAgent, flags.DisableAPI, false)
 	if err != nil {
 		return nil, err
 	}
 
-	if err = LoadConfig(cConfig); err != nil {
-		return nil, err
-	}
-	// Configure logging
-	if err = types.SetDefaultLoggerConfig(cConfig.Common.LogMedia,
-		cConfig.Common.LogDir, *cConfig.Common.LogLevel,
-		cConfig.Common.LogMaxSize, cConfig.Common.LogMaxFiles,
-		cConfig.Common.LogMaxAge, cConfig.Common.CompressLogs,
-		cConfig.Common.ForceColorLogs); err != nil {
-		return nil, err
-	}
-
 	if !cConfig.DisableAPI {
 		if flags.DisableCAPI {
 			log.Warningf("Communication with CrowdSec Central API disabled from args")
@@ -123,7 +111,7 @@ func ShutdownCrowdsecRoutines() error {
 	if len(dataSources) > 0 {
 		acquisTomb.Kill(nil)
 		log.Debugf("waiting for acquisition to finish")
-
+		drainChan(inputLineChan)
 		if err := acquisTomb.Wait(); err != nil {
 			log.Warningf("Acquisition returned error : %s", err)
 			reterr = err
@@ -132,6 +120,7 @@ func ShutdownCrowdsecRoutines() error {
 
 	log.Debugf("acquisition is finished, wait for parser/bucket/ouputs.")
 	parsersTomb.Kill(nil)
+	drainChan(inputEventChan)
 	if err := parsersTomb.Wait(); err != nil {
 		log.Warningf("Parsers returned error : %s", err)
 		reterr = err
@@ -202,6 +191,23 @@ func shutdown(sig os.Signal, cConfig *csconfig.Config) error {
 	return nil
 }
 
+func drainChan(c chan types.Event) {
+	time.Sleep(500 * time.Millisecond)
+	// delay to avoid draining chan before the acquisition/parser
+	// get a chance to push its event
+	// We should close the chan on the writer side rather than this
+	for {
+		select {
+		case _, ok := <-c:
+			if !ok { //closed
+				return
+			}
+		default:
+			return
+		}
+	}
+}
+
 func HandleSignals(cConfig *csconfig.Config) error {
 	var (
 		newConfig *csconfig.Config
@@ -292,6 +298,13 @@ func Serve(cConfig *csconfig.Config, apiReady chan bool, agentReady chan bool) e
 		log.Warningln("Exprhelpers loaded without database client.")
 	}
 
+	if cConfig.API.CTI != nil && *cConfig.API.CTI.Enabled {
+		log.Infof("Crowdsec CTI helper enabled")
+		if err := exprhelpers.InitCrowdsecCTI(cConfig.API.CTI.Key, cConfig.API.CTI.CacheTimeout, cConfig.API.CTI.CacheSize, cConfig.API.CTI.LogLevel); err != nil {
+			return errors.Wrap(err, "failed to init crowdsec cti")
+		}
+	}
+
 	if !cConfig.DisableAPI {
 		if cConfig.API.Server.OnlineClient == nil || cConfig.API.Server.OnlineClient.Credentials == nil {
 			log.Warningf("Communication with CrowdSec Central API disabled from configuration file")
@@ -339,19 +352,28 @@ func Serve(cConfig *csconfig.Config, apiReady chan bool, agentReady chan bool) e
 		if !sent || err != nil {
 			log.Errorf("Failed to notify(sent: %v): %v", sent, err)
 		}
-
 		// wait for signals
 		return HandleSignals(cConfig)
 	}
 
-	for {
-		select {
-		case <-apiTomb.Dead():
+	waitChans := make([]<-chan struct{}, 0)
+
+	if !cConfig.DisableAgent {
+		waitChans = append(waitChans, crowdsecTomb.Dead())
+	}
+
+	if !cConfig.DisableAPI {
+		waitChans = append(waitChans, apiTomb.Dead())
+	}
+
+	for _, ch := range waitChans {
+		<-ch
+		switch ch {
+		case apiTomb.Dead():
 			log.Infof("api shutdown")
-			return nil
-		case <-crowdsecTomb.Dead():
+		case crowdsecTomb.Dead():
 			log.Infof("crowdsec shutdown")
-			return nil
 		}
 	}
+	return nil
 }
