@@ -69,9 +69,10 @@ conf_set() {
     else
         YAML_FILE="$CONFIG_FILE"
     fi
-    YAML_CONTENT=$(cat "$YAML_FILE" 2>/dev/null || true)
-    NEW_CONTENT=$(echo "$YAML_CONTENT" | yq e "$1")
-    echo "$NEW_CONTENT" | install -m 0600 /dev/stdin "$YAML_FILE"
+    if [ ! -f "$YAML_FILE" ]; then
+        install -m 0600 /dev/null "$YAML_FILE"
+    fi
+    yq e "$1" -i "$YAML_FILE"
 }
 
 # conf_set_if(): used to update the configuration
@@ -105,11 +106,41 @@ cscli_if_clean() {
         if cscli "$1" inspect "$obj" -o json | yq -e '.tainted // false' >/dev/null 2>&1; then
             echo "Object $1/$obj is tainted, skipping"
         else
-            cscli "$1" "$2" "$obj"
+#            # Too verbose? Only show errors if not in debug mode
+#            if [ "$DEBUG" != "true" ]; then
+#                error_only=--error
+#            fi
+            error_only=""
+            echo "Running: cscli $error_only $1 $2 \"$obj\""
+            # shellcheck disable=SC2086
+            cscli $error_only "$1" "$2" "$obj"
         fi
     done
 }
 
+# Output the difference between two lists
+# of items separated by spaces
+difference() {
+  list1="$1"
+  list2="$2"
+
+  # split into words
+  # shellcheck disable=SC2086
+  set -- $list1
+  for item in "$@"; do
+    found=false
+    for i in $list2; do
+      if [ "$item" = "$i" ]; then
+        found=true
+        break
+      fi
+    done
+    if [ "$found" = false ]; then
+      echo "$item"
+    fi
+  done
+}
+
 #-----------------------------------#
 
 if [ -n "$CERT_FILE" ] || [ -n "$KEY_FILE" ] ; then
@@ -118,8 +149,8 @@ if [ -n "$CERT_FILE" ] || [ -n "$KEY_FILE" ] ; then
     echo "Please use LAPI_CERT_FILE and LAPI_KEY_FILE insted." >&2
     echo "The old variables will be removed in a future release." >&2
     printf '%b' '\033[0m'
-    LAPI_CERT_FILE=${LAPI_CERT_FILE:-$CERT_FILE}
-    LAPI_KEY_FILE=${LAPI_KEY_FILE:-$KEY_FILE}
+    export LAPI_CERT_FILE=${LAPI_CERT_FILE:-$CERT_FILE}
+    export LAPI_KEY_FILE=${LAPI_KEY_FILE:-$KEY_FILE}
 fi
 
 # Check and prestage databases
@@ -154,13 +185,22 @@ elif [ -n "$USE_WAL" ] && isfalse "$USE_WAL"; then
     conf_set '.db_config.use_wal = false'
 fi
 
-# regenerate local agent credentials (even if agent is disabled, cscli needs a
-# connection to the API)
-cscli machines delete "$CUSTOM_HOSTNAME" 2>/dev/null || true
+lapi_credentials_path=$(conf_get '.api.client.credentials_path')
+
+
 if isfalse "$DISABLE_LOCAL_API"; then
-    if isfalse "$USE_TLS" || [ "$CLIENT_CERT_FILE" = "" ]; then
-        echo "Regenerate local agent credentials"
-        cscli machines add "$CUSTOM_HOSTNAME" --auto
+    # generate local agent credentials (even if agent is disabled, cscli needs a
+    # connection to the API)
+    if ( isfalse "$USE_TLS" || [ "$CLIENT_CERT_FILE" = "" ] ); then
+        if yq -e '.login==strenv(CUSTOM_HOSTNAME)' "$lapi_credentials_path" >/dev/null && ( cscli machines list -o json | yq -e 'any_c(.machineId==strenv(CUSTOM_HOSTNAME))' >/dev/null ); then
+            echo "Local agent already registered"
+        else
+            echo "Generate local agent credentials"
+            # if the db is persistent but the credentials are not, we need to
+            # delete the old machine to generate new credentials
+            cscli machines delete "$CUSTOM_HOSTNAME" >/dev/null 2>&1 || true
+            cscli machines add "$CUSTOM_HOSTNAME" --auto
+        fi
     fi
 
     echo "Check if lapi needs to register an additional agent"
@@ -174,8 +214,6 @@ fi
 
 # ----------------
 
-lapi_credentials_path=$(conf_get '.api.client.credentials_path')
-
 conf_set_if "$LOCAL_API_URL" '.url = strenv(LOCAL_API_URL)' "$lapi_credentials_path"
 
 if istrue "$DISABLE_LOCAL_API"; then
@@ -208,9 +246,9 @@ fi
 # registration to online API for signal push
 if isfalse "$DISABLE_ONLINE_API" ; then
     CONFIG_DIR=$(conf_get '.config_paths.config_dir')
+    export CONFIG_DIR
     config_exists=$(conf_get '.api.server.online_client | has("credentials_path")')
     if isfalse "$config_exists"; then
-        export CONFIG_DIR
         conf_set '.api.server.online_client = {"credentials_path": strenv(CONFIG_DIR) + "/online_api_credentials.yaml"}'
         cscli capi register > "$CONFIG_DIR/online_api_credentials.yaml"
         echo "Registration to online API done"
@@ -243,8 +281,10 @@ fi
 
 # XXX only with LAPI
 if istrue "$USE_TLS"; then
-    agents_allowed_yaml=$(csv2yaml "$AGENTS_ALLOWED_OU") \
-    bouncers_allowed_yaml=$(csv2yaml "$BOUNCERS_ALLOWED_OU") \
+    agents_allowed_yaml=$(csv2yaml "$AGENTS_ALLOWED_OU")
+    export agents_allowed_yaml
+    bouncers_allowed_yaml=$(csv2yaml "$BOUNCERS_ALLOWED_OU")
+    export bouncers_allowed_yaml
     conf_set_if "$CACERT_FILE" '.api.server.tls.ca_cert_path = strenv(CACERT_FILE)'
     conf_set_if "$LAPI_CERT_FILE" '.api.server.tls.cert_file = strenv(LAPI_CERT_FILE)'
     conf_set_if "$LAPI_KEY_FILE" '.api.server.tls.key_file = strenv(LAPI_KEY_FILE)'
@@ -262,25 +302,26 @@ cscli hub update
 cscli_if_clean collections upgrade crowdsecurity/linux
 cscli_if_clean parsers upgrade crowdsecurity/whitelists
 cscli_if_clean parsers install crowdsecurity/docker-logs
+cscli_if_clean parsers install crowdsecurity/cri-logs
 
 if [ "$COLLECTIONS" != "" ]; then
     # shellcheck disable=SC2086
-    cscli_if_clean collections install "$COLLECTIONS"
+    cscli_if_clean collections install "$(difference "$COLLECTIONS" "$DISABLE_COLLECTIONS")"
 fi
 
 if [ "$PARSERS" != "" ]; then
     # shellcheck disable=SC2086
-    cscli_if_clean parsers install "$PARSERS"
+    cscli_if_clean parsers install "$(difference "$PARSERS" "$DISABLE_PARSERS")"
 fi
 
 if [ "$SCENARIOS" != "" ]; then
     # shellcheck disable=SC2086
-    cscli_if_clean scenarios install "$SCENARIOS"
+    cscli_if_clean scenarios install "$(difference "$SCENARIOS" "$DISABLE_SCENARIOS")"
 fi
 
 if [ "$POSTOVERFLOWS" != "" ]; then
     # shellcheck disable=SC2086
-    cscli_if_clean postoverflows install "$POSTOVERFLOWS"
+    cscli_if_clean postoverflows install "$(difference "$POSTOVERFLOWS" "$DISABLE_POSTOVERFLOWS")"
 fi
 
 ## Remove collections, parsers, scenarios & postoverflows
@@ -313,7 +354,7 @@ for BOUNCER in $(compgen -A variable | grep -i BOUNCER_KEY); do
     fi
 done
 
-## Register bouncers via secrets
+## Register bouncers via secrets (Swarm only)
 shopt -s nullglob extglob
 for BOUNCER in /run/secrets/@(bouncer_key|BOUNCER_KEY)* ; do
     KEY=$(cat "${BOUNCER}")
