@@ -27,7 +27,7 @@ func BuildDecisionRequestWithFilter(query *ent.DecisionQuery, filter map[string]
 	var err error
 	var start_ip, start_sfx, end_ip, end_sfx int64
 	var ip_sz int
-	var contains bool = true
+	var contains = true
 	/*if contains is true, return bans that *contains* the given value (value is the inner)
 	  else, return bans that are *contained* by the given value (value is the outer)*/
 
@@ -86,6 +86,24 @@ func BuildDecisionRequestWithFilter(query *ent.DecisionQuery, filter map[string]
 			if err != nil {
 				return nil, errors.Wrapf(InvalidIPOrRange, "unable to convert '%s' to int: %s", value[0], err)
 			}
+		case "limit":
+			limit, err := strconv.Atoi(value[0])
+			if err != nil {
+				return nil, errors.Wrapf(InvalidFilter, "invalid limit value : %s", err)
+			}
+			query = query.Limit(limit)
+		case "offset":
+			offset, err := strconv.Atoi(value[0])
+			if err != nil {
+				return nil, errors.Wrapf(InvalidFilter, "invalid offset value : %s", err)
+			}
+			query = query.Offset(offset)
+		case "id_gt":
+			id, err := strconv.Atoi(value[0])
+			if err != nil {
+				return nil, errors.Wrapf(InvalidFilter, "invalid id_gt value : %s", err)
+			}
+			query = query.Where(decision.IDGT(id))
 		}
 	}
 	query, err = applyStartIpEndIpFilter(query, contains, ip_sz, start_ip, start_sfx, end_ip, end_sfx)
@@ -110,6 +128,8 @@ func (c *Client) QueryAllDecisionsWithFilters(filters map[string][]string) ([]*e
 		return []*ent.Decision{}, errors.Wrap(QueryFail, "get all decisions with filters")
 	}
 
+	query = query.Order(ent.Asc(decision.FieldID))
+
 	data, err := query.All(c.CTX)
 	if err != nil {
 		c.Log.Warningf("QueryAllDecisionsWithFilters : %s", err)
@@ -129,6 +149,8 @@ func (c *Client) QueryExpiredDecisionsWithFilters(filters map[string][]string) (
 
 	query, err := BuildDecisionRequestWithFilter(query, filters)
 
+	query = query.Order(ent.Asc(decision.FieldID))
+
 	if err != nil {
 		c.Log.Warningf("QueryExpiredDecisionsWithFilters : %s", err)
 		return []*ent.Decision{}, errors.Wrap(QueryFail, "get expired decisions with filters")
@@ -238,6 +260,8 @@ func (c *Client) QueryExpiredDecisionsSinceWithFilters(since time.Time, filters
 		return []*ent.Decision{}, errors.Wrap(QueryFail, "expired decisions with filters")
 	}
 
+	query = query.Order(ent.Asc(decision.FieldID))
+
 	data, err := query.All(c.CTX)
 	if err != nil {
 		c.Log.Warningf("QueryExpiredDecisionsSinceWithFilters : %s", err)
@@ -261,6 +285,9 @@ func (c *Client) QueryNewDecisionsSinceWithFilters(since time.Time, filters map[
 		c.Log.Warningf("QueryNewDecisionsSinceWithFilters : %s", err)
 		return []*ent.Decision{}, errors.Wrapf(QueryFail, "new decisions since '%s'", since.String())
 	}
+
+	query = query.Order(ent.Asc(decision.FieldID))
+
 	data, err := query.All(c.CTX)
 	if err != nil {
 		c.Log.Warningf("QueryNewDecisionsSinceWithFilters : %s", err)
@@ -269,30 +296,32 @@ func (c *Client) QueryNewDecisionsSinceWithFilters(since time.Time, filters map[
 	return data, nil
 }
 
-func (c *Client) DeleteDecisionById(decisionId int) error {
-	err := c.Ent.Decision.DeleteOneID(decisionId).Exec(c.CTX)
+func (c *Client) DeleteDecisionById(decisionId int) ([]*ent.Decision, error) {
+	toDelete, err := c.Ent.Decision.Query().Where(decision.IDEQ(decisionId)).All(c.CTX)
 	if err != nil {
 		c.Log.Warningf("DeleteDecisionById : %s", err)
-		return errors.Wrapf(DeleteFail, "decision with id '%d' doesn't exist", decisionId)
+		return nil, errors.Wrapf(DeleteFail, "decision with id '%d' doesn't exist", decisionId)
 	}
-	return nil
+	count, err := c.BulkDeleteDecisions(toDelete, false)
+	c.Log.Debugf("deleted %d decisions", count)
+	return toDelete, err
 }
 
-func (c *Client) DeleteDecisionsWithFilter(filter map[string][]string) (string, error) {
+func (c *Client) DeleteDecisionsWithFilter(filter map[string][]string) (string, []*ent.Decision, error) {
 	var err error
 	var start_ip, start_sfx, end_ip, end_sfx int64
 	var ip_sz int
-	var contains bool = true
+	var contains = true
 	/*if contains is true, return bans that *contains* the given value (value is the inner)
 	  else, return bans that are *contained* by the given value (value is the outer) */
 
-	decisions := c.Ent.Decision.Delete()
+	decisions := c.Ent.Decision.Query()
 	for param, value := range filter {
 		switch param {
 		case "contains":
 			contains, err = strconv.ParseBool(value[0])
 			if err != nil {
-				return "0", errors.Wrapf(InvalidFilter, "invalid contains value : %s", err)
+				return "0", nil, errors.Wrapf(InvalidFilter, "invalid contains value : %s", err)
 			}
 		case "scope":
 			decisions = decisions.Where(decision.ScopeEQ(value[0]))
@@ -303,12 +332,12 @@ func (c *Client) DeleteDecisionsWithFilter(filter map[string][]string) (string,
 		case "ip", "range":
 			ip_sz, start_ip, start_sfx, end_ip, end_sfx, err = types.Addr2Ints(value[0])
 			if err != nil {
-				return "0", errors.Wrapf(InvalidIPOrRange, "unable to convert '%s' to int: %s", value[0], err)
+				return "0", nil, errors.Wrapf(InvalidIPOrRange, "unable to convert '%s' to int: %s", value[0], err)
 			}
 		case "scenario":
 			decisions = decisions.Where(decision.ScenarioEQ(value[0]))
 		default:
-			return "0", errors.Wrap(InvalidFilter, fmt.Sprintf("'%s' doesn't exist", param))
+			return "0", nil, errors.Wrap(InvalidFilter, fmt.Sprintf("'%s' doesn't exist", param))
 		}
 	}
 
@@ -377,35 +406,42 @@ func (c *Client) DeleteDecisionsWithFilter(filter map[string][]string) (string,
 			))
 		}
 	} else if ip_sz != 0 {
-		return "0", errors.Wrapf(InvalidFilter, "Unknown ip size %d", ip_sz)
+		return "0", nil, errors.Wrapf(InvalidFilter, "Unknown ip size %d", ip_sz)
 	}
 
-	nbDeleted, err := decisions.Exec(c.CTX)
+	toDelete, err := decisions.All(c.CTX)
 	if err != nil {
 		c.Log.Warningf("DeleteDecisionsWithFilter : %s", err)
-		return "0", errors.Wrap(DeleteFail, "decisions with provided filter")
+		return "0", nil, errors.Wrap(DeleteFail, "decisions with provided filter")
 	}
-	return strconv.Itoa(nbDeleted), nil
+	count, err := c.BulkDeleteDecisions(toDelete, false)
+	if err != nil {
+		c.Log.Warningf("While deleting decisions : %s", err)
+		return "0", nil, errors.Wrap(DeleteFail, "decisions with provided filter")
+	}
+	return strconv.Itoa(count), toDelete, nil
 }
 
-// SoftDeleteDecisionsWithFilter updates the expiration time to now() for the decisions matching the filter
-func (c *Client) SoftDeleteDecisionsWithFilter(filter map[string][]string) (string, error) {
+// SoftDeleteDecisionsWithFilter updates the expiration time to now() for the decisions matching the filter, and returns the updated items
+func (c *Client) SoftDeleteDecisionsWithFilter(filter map[string][]string) (string, []*ent.Decision, error) {
 	var err error
 	var start_ip, start_sfx, end_ip, end_sfx int64
 	var ip_sz int
-	var contains bool = true
+	var contains = true
 	/*if contains is true, return bans that *contains* the given value (value is the inner)
 	  else, return bans that are *contained* by the given value (value is the outer)*/
-	decisions := c.Ent.Decision.Update().Where(decision.UntilGT(time.Now().UTC()))
+	decisions := c.Ent.Decision.Query().Where(decision.UntilGT(time.Now().UTC()))
 	for param, value := range filter {
 		switch param {
 		case "contains":
 			contains, err = strconv.ParseBool(value[0])
 			if err != nil {
-				return "0", errors.Wrapf(InvalidFilter, "invalid contains value : %s", err)
+				return "0", nil, errors.Wrapf(InvalidFilter, "invalid contains value : %s", err)
 			}
 		case "scopes":
 			decisions = decisions.Where(decision.ScopeEQ(value[0]))
+		case "uuid":
+			decisions = decisions.Where(decision.UUIDIn(value...))
 		case "origin":
 			decisions = decisions.Where(decision.OriginEQ(value[0]))
 		case "value":
@@ -415,12 +451,12 @@ func (c *Client) SoftDeleteDecisionsWithFilter(filter map[string][]string) (stri
 		case "ip", "range":
 			ip_sz, start_ip, start_sfx, end_ip, end_sfx, err = types.Addr2Ints(value[0])
 			if err != nil {
-				return "0", errors.Wrapf(InvalidIPOrRange, "unable to convert '%s' to int: %s", value[0], err)
+				return "0", nil, errors.Wrapf(InvalidIPOrRange, "unable to convert '%s' to int: %s", value[0], err)
 			}
 		case "scenario":
 			decisions = decisions.Where(decision.ScenarioEQ(value[0]))
 		default:
-			return "0", errors.Wrapf(InvalidFilter, "'%s' doesn't exist", param)
+			return "0", nil, errors.Wrapf(InvalidFilter, "'%s' doesn't exist", param)
 		}
 	}
 	if ip_sz == 4 {
@@ -492,28 +528,89 @@ func (c *Client) SoftDeleteDecisionsWithFilter(filter map[string][]string) (stri
 			))
 		}
 	} else if ip_sz != 0 {
-		return "0", errors.Wrapf(InvalidFilter, "Unknown ip size %d", ip_sz)
+		return "0", nil, errors.Wrapf(InvalidFilter, "Unknown ip size %d", ip_sz)
 	}
-	nbDeleted, err := decisions.SetUntil(time.Now().UTC()).Save(c.CTX)
+	DecisionsToDelete, err := decisions.All(c.CTX)
 	if err != nil {
 		c.Log.Warningf("SoftDeleteDecisionsWithFilter : %s", err)
-		return "0", errors.Wrap(DeleteFail, "soft delete decisions with provided filter")
+		return "0", nil, errors.Wrap(DeleteFail, "soft delete decisions with provided filter")
 	}
-	return strconv.Itoa(nbDeleted), nil
+
+	count, err := c.BulkDeleteDecisions(DecisionsToDelete, true)
+	if err != nil {
+		return "0", nil, errors.Wrapf(DeleteFail, "soft delete decisions with provided filter : %s", err)
+	}
+	return strconv.Itoa(count), DecisionsToDelete, err
+}
+
+// BulkDeleteDecisions set the expiration of a bulk of decisions to now() or hard deletes them.
+// We are doing it this way so we can return impacted decisions for sync with CAPI/PAPI
+func (c *Client) BulkDeleteDecisions(DecisionsToDelete []*ent.Decision, softDelete bool) (int, error) {
+	bulkSize := 256 //scientifically proven to be the best value for bulk delete
+	idsToDelete := make([]int, 0, bulkSize)
+	totalUpdates := 0
+	for i := 0; i < len(DecisionsToDelete); i++ {
+		idsToDelete = append(idsToDelete, DecisionsToDelete[i].ID)
+		if len(idsToDelete) == bulkSize {
+
+			if softDelete {
+				nbUpdates, err := c.Ent.Decision.Update().Where(
+					decision.IDIn(idsToDelete...),
+				).SetUntil(time.Now().UTC()).Save(c.CTX)
+				if err != nil {
+					return totalUpdates, errors.Wrap(err, "soft delete decisions with provided filter")
+				}
+				totalUpdates += nbUpdates
+			} else {
+				nbUpdates, err := c.Ent.Decision.Delete().Where(
+					decision.IDIn(idsToDelete...),
+				).Exec(c.CTX)
+				if err != nil {
+					return totalUpdates, errors.Wrap(err, "hard delete decisions with provided filter")
+				}
+				totalUpdates += nbUpdates
+			}
+			idsToDelete = make([]int, 0, bulkSize)
+		}
+	}
+
+	if len(idsToDelete) > 0 {
+		if softDelete {
+			nbUpdates, err := c.Ent.Decision.Update().Where(
+				decision.IDIn(idsToDelete...),
+			).SetUntil(time.Now().UTC()).Save(c.CTX)
+			if err != nil {
+				return totalUpdates, errors.Wrap(err, "soft delete decisions with provided filter")
+			}
+			totalUpdates += nbUpdates
+		} else {
+			nbUpdates, err := c.Ent.Decision.Delete().Where(
+				decision.IDIn(idsToDelete...),
+			).Exec(c.CTX)
+			if err != nil {
+				return totalUpdates, errors.Wrap(err, "hard delete decisions with provided filter")
+			}
+			totalUpdates += nbUpdates
+		}
+
+	}
+	return totalUpdates, nil
 }
 
 // SoftDeleteDecisionByID set the expiration of a decision to now()
-func (c *Client) SoftDeleteDecisionByID(decisionID int) (int, error) {
-	nbUpdated, err := c.Ent.Decision.Update().Where(decision.IDEQ(decisionID)).SetUntil(time.Now().UTC()).Save(c.CTX)
-	if err != nil || nbUpdated == 0 {
-		c.Log.Warningf("SoftDeleteDecisionByID : %v (nb soft deleted: %d)", err, nbUpdated)
-		return 0, errors.Wrapf(DeleteFail, "decision with id '%d' doesn't exist", decisionID)
+func (c *Client) SoftDeleteDecisionByID(decisionID int) (int, []*ent.Decision, error) {
+	toUpdate, err := c.Ent.Decision.Query().Where(decision.IDEQ(decisionID)).All(c.CTX)
+
+	if err != nil || len(toUpdate) == 0 {
+		c.Log.Warningf("SoftDeleteDecisionByID : %v (nb soft deleted: %d)", err, len(toUpdate))
+		return 0, nil, errors.Wrapf(DeleteFail, "decision with id '%d' doesn't exist", decisionID)
 	}
 
-	if nbUpdated == 0 {
-		return 0, ItemNotFound
+	if len(toUpdate) == 0 {
+		return 0, nil, ItemNotFound
 	}
-	return nbUpdated, nil
+	count, err := c.BulkDeleteDecisions(toUpdate, true)
+	return count, toUpdate, err
 }
 
 func (c *Client) CountDecisionsByValue(decisionValue string) (int, error) {
