@@ -7,13 +7,16 @@ import (
 
 var pathCache = make(map[string]etree.Path)
 
-func XMLGetAttributeValue(xmlString string, path string, attributeName string) string {
-
+// func XMLGetAttributeValue(xmlString string, path string, attributeName string) string {
+func XMLGetAttributeValue(params ...any) (any, error) {
+	xmlString := params[0].(string)
+	path := params[1].(string)
+	attributeName := params[2].(string)
 	if _, ok := pathCache[path]; !ok {
 		compiledPath, err := etree.CompilePath(path)
 		if err != nil {
 			log.Errorf("Could not compile path %s: %s", path, err)
-			return ""
+			return "", nil
 		}
 		pathCache[path] = compiledPath
 	}
@@ -23,27 +26,30 @@ func XMLGetAttributeValue(xmlString string, path string, attributeName string) s
 	err := doc.ReadFromString(xmlString)
 	if err != nil {
 		log.Tracef("Could not parse XML: %s", err)
-		return ""
+		return "", nil
 	}
 	elem := doc.FindElementPath(compiledPath)
 	if elem == nil {
 		log.Debugf("Could not find element %s", path)
-		return ""
+		return "", nil
 	}
 	attr := elem.SelectAttr(attributeName)
 	if attr == nil {
 		log.Debugf("Could not find attribute %s", attributeName)
-		return ""
+		return "", nil
 	}
-	return attr.Value
+	return attr.Value, nil
 }
 
-func XMLGetNodeValue(xmlString string, path string) string {
+// func XMLGetNodeValue(xmlString string, path string) string {
+func XMLGetNodeValue(params ...any) (any, error) {
+	xmlString := params[0].(string)
+	path := params[1].(string)
 	if _, ok := pathCache[path]; !ok {
 		compiledPath, err := etree.CompilePath(path)
 		if err != nil {
 			log.Errorf("Could not compile path %s: %s", path, err)
-			return ""
+			return "", nil
 		}
 		pathCache[path] = compiledPath
 	}
@@ -53,12 +59,12 @@ func XMLGetNodeValue(xmlString string, path string) string {
 	err := doc.ReadFromString(xmlString)
 	if err != nil {
 		log.Tracef("Could not parse XML: %s", err)
-		return ""
+		return "", nil
 	}
 	elem := doc.FindElementPath(compiledPath)
 	if elem == nil {
 		log.Debugf("Could not find element %s", path)
-		return ""
+		return "", nil
 	}
-	return elem.Text()
+	return elem.Text(), nil
 }
